########################################################################
# Module web2ldapcnf.py
# (c) by Michael Stroeder, michael.stroeder@inka.de
# Note: This module has to reside in the cgi-bin directory
########################################################################

Version='0.7.5'

########################################################################
# Some variables to configure the basic behaviour of web2ldap.py
# All code must be valid Python...
########################################################################

# A list with directories containing additional Python modules
pylibdirs = ['/usr/lib/web2ldap/pylib','../pylib']


########################################################################
# Options for running as stand-alone web server
########################################################################

# Where static documents reside
document_root = '/usr/lib/web2ldap/html/'

# Path name of mime.types file
#mime_types = '/etc/mime.types'
mime_types = ''

# Contact mail address of the server admin when running stand-alone
server_admin = 'ldapadmin@stp-online.de'

# Server signature when running stand-alone, HTML snippet,
server_signature = """<ADDRESS>%(SERVER_SOFTWARE)s at
<A HREF="mailto:%(SERVER_ADMIN)s">%(SERVER_NAME)s</A>
Port %(SERVER_PORT)s</ADDRESS>
"""

########################################################################
# HTTP-Options
########################################################################

# UTF-8 data is printed as is and charset is set to utf-8 in HTTP header:
# 0 Never
# 1 if the browser sends utf-8 in HTTP header and the web server sets
#   HTTP_ACCEPT_CHARSET environment variable
# 2 always
print_rawutf8=1

# Maximum length of LDIF data in the <TEXTAREA> of addform/modifyform
ldif_maxbytes=200000

# Maximum count of input attribute fields in addform/modifyform
input_maxattrs=200

# There are some situations where this client just wants to get the
# attributes of an entry and not the data itself for saving bandwidth.
# However some LDAP hosts (e.g. Notes Domino 4.61) have problems with
# such an attribute-only request, they won't return any matches for a search.
# If you experience this problems (no matching entry) set this to 0.
search_attrsonly=1

# Known attributes for search form
search_attr = {
  'Country':'c',
  'State':'st',
  'Location':'l',
  'Organization':'o',
  'Organizational Unit':'ou',
  'Common Name':'cn',
  'Surname':'sn',
  'Given Name':'givenname',
  'Title':'title',
  'Street':'streetaddress',
  'E-Mail':'mail',
  'User ID':'uid',
  'Initials':'initials',
  'Object Class':'objectclass',
  'Telephone':'telephonenumber',
  'FAX':'facsimiletelephonenumber',
  'Home phone':'homephone',
  'Description':'description'
}

# This sets the mapping of LDAP to vCard attributes
# See http://developer.netscape.com:80/docs/manuals/dsgw_html/dsgwparm.htm
vcard_property = {
  'fn':('cis',['cn']),
  'n':('cis',['sn','givenname']),
  'tel;pager':('cis',['mobile']),
  'tel;cell':('cis',['mobile']),
  'tel;fax':('cis',['facsimiletelephonenumber']),
  'tel;home':('cis',['homephone']),
  'tel;work':('cis',['telephonenumber']),
  'url':('cis',['labeleduri']),
  'org':('cis',['o','ou']),
  'email;internet':('cis',['mail']),
  'title':('cis',['title']),
  'role':('cis',['role']),
  'adr':('mls',['streetaddress','l','postalcode']),
  'note':('mls',['description'])
}

# maximum count of search parameters in 'searchform'
max_searchparams=9

# map binary attributes to MIME-types
ldap_binaryattr = {
    'cacertificate;binary':('CA certificate','application/x-x509-ca-cert'),
    'certificaterevocationlist;binary':('User CRL','application/x-pkcs7-crl'),
    'certificate;binary':('Client certificate','application/x-x509-email-cert'),
    'usercertificate;binary':('Client certificate','application/x-x509-email-cert'),
    'usersmimecertificate;binary':('S/MIME certificate','application/x-x509-email-cert'),
    'certificate':('Client certificate','application/x-x509-email-cert'),
    'usercertificate':('Client certificate','application/x-x509-email-cert'),
    'usersmimecertificate':('S/MIME certificate','application/x-x509-email-cert'),
    'jpegphoto':('Photograph','image/jpeg'),
    'thumbnailphoto':('Photograph','image/jpeg'),
    'photo':('Photograph','image/jpeg'),
    'userpkcs12;binary':('PKCS#12','application/pkcs12'),
    'audio':('Audio','audio/basic')
  }

# Some browsers are more equal than others ;-)
ldap_browsermimetypes = {
    'certificaterevocationlist;binary': {
      'MSIE':'application/pkix-crl'
    }
  }

# Additional information in <HEAD></HEAD> section, e.g. CSS.
# Do not put <TITLE> tags in here!
#html_head = r"""
#  <LINK rel=stylesheet type="text/css" href="http://sites.inka.de/ms/css/web2ldap.css">
#"""
html_head = r"""

"""

# Rudimental configuration for <BODY>-tag
#html_bodybegin = r"""
#<BODY LINK="Red" VLINK="Maroon" BGCOLOR="#fffffe">
#"""
html_bodybegin = r"""
<BODY>
"""

########################################################################
# Security options
########################################################################

# This is the HTTP method for submitting forms
# Use POST to avoid security problems with usernames and passwords
# to be stored in URLs (-> web server logs). Also POST can
# handle more input data.
# GET is quite handy for debugging and works reliable in stand-alone mode
# on all platforms.
script_method = 'POST'
#script_method = 'GET'

# If sec_expire is 0 there will be sent 'Pragma: no-cache' in the header
# which is suitable in situations where you modify a lot through web2ldap.
# However this might be quite uncomfortable for the user while pressing the
# browser's back button. You can set sec_expire to define the amount of
# time (in seconds) the data is valid in the browser.
# Note that if you're doing a lot of modifications you might get
# an nasty behaviour if the pages are cached by your browser.
sec_expire = 0

# You can specify the required security level for each host and each command.
# This default is chosen if there's no definition for a specific
# host and command.
# Set this to 2 if you are using SSL web server with client certificates.
# Set to 1 if you use SSL connections.
# Set to 0 if all LDAP data you are handling through this gateway is public.
sec_reqlevel_default = 0

# Specifies a list of the acceptable symmetric key ciphers to
# reach security level 1.
# See also http://www.apache-ssl.org/docs.html and the ApacheSSL
# run-time directives SSLBanCipher, SSLRequireCipher, SSLRequiredCiphers
sec_sslacceptedciphers = [
# Strong ciphers
    'IDEA-CBC-SHA',
    'RC4-MD5',
    'RC4-SHA',
    'IDEA-CBC-MD5',
    'DES-CBC3-SHA',
    'DH-DSS-DES-CBC3-SHA',
    'DH-RSA-DES-CBC3-SHA',
    'EDH-DSS-DES-CBC3-SHA',
    'EDH-RSA-DES-CBC3-SHA',
    'ADH-RC4-MD5',
    'ADH-DES-CBC3-SHA',
    'FZA-RC4-SHA',
    'RC2-CBC-MD5',
    'DES-CBC3-MD5',
# Weak export ciphers
#    'EXP-RC4-MD5',
#    'EXP-RC2-CBC-MD5',
#    'EXP-DES-CBC-SHA',
#    'DES-CBC-SHA',
#    'EXP-DH-DSS-DES-CBC-SHA',
#    'DH-DSS-DES-CBC-SHA',
#    'EXP-DH-RSA-DES-CBC-SHA',
#    'DH-RSA-DES-CBC-SHA',
#    'EXP-EDH-DSS-DES-CBC-SHA',
#    'EDH-DSS-DES-CBC-SHA',
#    'EXP-EDH-RSA-DES-CBC',
#    'EDH-RSA-DES-CBC-SHA',
#    'EXP-ADH-RC4-MD5',
#    'EXP-ADH-DES-CBC-SHA',
#    'ADH-DES-CBC-SHA',
#    'DES-CFB-M1',
#    'DES-CBC-MD5',
#    'RC4-64-MD5'
  ]

########################################################################
# Define LDAP hosts used in web2ldap.py
########################################################################

# List of all LDAP hosts to use. These hosts will appear in the
# default select list of the login form.
# A list containing only one host results in a normal input field
# with the host set as default.
ldap_host_list = [
                   'ldap',
                   'ldap.ms.inka.de',
                   'queen.ms.inka.de',
                   'ldap.tu-chemnitz.de',
		   'nldap.com',
		   'ldap.novell.com',
		   'ldap.surfnet.nl',
		   'sun02.directory.dfn.de:17389',
		   'db.debian.org',
		   'elvira.innosoft.com'
		 ]


########################################################################
# LDAP host(s) with their defaults can be pre-defined as dictionary
# ldap_def = {'host:port':{paramdictionary}}
########################################################################

ldap_def = {
  'ldap.novell.com': {
    # Base DN for this host
    'basedn':'',
    # Template for bind-DN
    'binddn':r'',
    # Template for searching bind-DN
    'binddnsearch':r'(|(cn=*%s*)(mail=%s*)(initials=%s))',
    # Regular expression for valid client cert DNs
    'sec_valid_dn':r'/.*',
    # Regular expression for valid issuer DNs of client certs
    'sec_valid_idn':r'/.*',
    # Required security level for each command {command:sec_ssllevel}
    # The sec_ssllevel is defined like this:
    # 0	no encryption at all
    # 1	session is encrypted with SSL and
    #       cipher is in sec_sslacceptedciphers (see below)
    # 2	client has valid certificate and
    # 	sec_valid_dn and sec_valid_idn are matched
    'sec_reqlevel': {
      'add':0,
      'delete':0,
      'modify':0,
      'passwd':0,
      'read':0,
      'vcard':0,
      'search':0,
      'print':0,
      'menu':0,
      'secinfo':0,
      'searchform':0,
      'filterform':0,
      'input':0
    },

    # LDAP-attributes which always should kept hidden from user.
    # This list should at least contain the attribute 'userPassword'!
    'hiddenattrs':['userpassword','modifytimestamp','modifiersname'],

    # These attributes with this order are listed in the table
    # of the search results. If set to empty list the DN is displayed.
    'search_tablistattrs':['givenname','sn','uid','mail','telephonenumber'],

    # This is the HTML template for the content of one table cell
    # Use a %(attr)s for each attribute.
    # (See Python's dictionary string formatting syntax)
    'print_template':r'<FONT SIZE=-3>%(givenname)s %(sn)s<BR>Tel.: %(telephonenumber)s<BR>FAX: %(facsimiletelephonenumber)s<BR>E-Mail: %(mail)s</FONT>',
    # Number of columns for pretty-printable report
    'print_cols':4
  },

  'memberdir.netscape.com': {
    'basedn':'ou=member_directory,o=netcenter.com',
    'binddn':r'',
    'binddnsearch':r'(|(cn=*%s*)(mail=%s*)(initials=%s))',
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    'search_tablistattrs':['cn','mail'],
    'print_template':r'<FONT SIZE=-3>%(givenname)s %(sn)s<BR>Tel.: %(telephonenumber)s<BR>FAX: %(facsimiletelephonenumber)s<BR>E-Mail: %(mail)s</FONT>',
    'print_cols':4
  },

  'ldap.four11.com': {
    'basedn':'',
    'binddn':r'',
    'binddnsearch':r'(|(cn=*%s*)(mail=%s*)(initials=%s))',
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    # 'search_tablistattrs':['cn','mail'],
    'print_template':r'<FONT SIZE=-3>%(givenname)s %(sn)s<BR>Tel.: %(telephonenumber)s<BR>FAX: %(facsimiletelephonenumber)s<BR>E-Mail: %(mail)s</FONT>',
    'print_cols':4
  },

  'ldap.infospace.com': {
    'basedn':'c=US',
    'binddn':r'',
    'binddnsearch':r'(|(cn=*%s*)(mail=%s*)(initials=%s))',
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    'search_tablistattrs':['cn','mail'],
    'print_template':r'<FONT SIZE=-3>%(givenname)s %(sn)s<BR>Tel.: %(telephonenumber)s<BR>FAX: %(facsimiletelephonenumber)s<BR>E-Mail: %(mail)s</FONT>',
    'print_cols':4
  },

  'directory.verisign.com': {
    'basedn':'',
    'binddn':r'',
    'binddnsearch':r'(|(cn=*%s*)(mail=%s*)(initials=%s))',
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    'search_tablistattrs':['cn','mail'],
    'print_template':r'<FONT SIZE=-3>%(givenname)s %(sn)s<BR>Tel.: %(telephonenumber)s<BR>FAX: %(facsimiletelephonenumber)s<BR>E-Mail: %(mail)s</FONT>',
    'print_cols':4
  },

  'sun02.directory.dfn.de:17389': {
    'basedn':'c=DE',
    'binddn':r'',
    'binddnsearch':r'',
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    'search_tablistattrs':['cn','mail'],
    'print_template':r'<FONT SIZE=-3>%(givenname)s %(sn)s<BR>Tel.: %(telephonenumber)s<BR>FAX: %(facsimiletelephonenumber)s<BR>E-Mail: %(mail)s</FONT>',
    'print_cols':4
  },

  'db.debian.org': {
    'basedn':'dc=debian,dc=org',
    'binddn':r'',
    'binddnsearch':r'',
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    'search_tablistattrs':['cn','mail'],
    'print_template':r'<FONT SIZE=-3>%(givenname)s %(sn)s<BR>Tel.: %(telephonenumber)s<BR>FAX: %(facsimiletelephonenumber)s<BR>E-Mail: %(mail)s</FONT>',
    'print_cols':4
  },

  'elvira.innosoft.com': {
    'basedn':'dc=Innosoft,dc=com',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    'search_tablistattrs':['cn','mail','homephone','telephonenumber'],
    'print_template':r'<FONT SIZE=-3>%(cn)s<BR>%(streetaddress)s<BR>%(l)s<BR>%(homephone)s<BR>%(telephonenumber)s<BR>%(mobile)s<BR>%(mail)s</FONT>',
    'print_cols':4
  },

  # To all hacker wanna-be's:
  # This not a public accessible LDAP host. Don't try.
  # Adjust the settings to reflect your local LDAP installation.
  'ldap.ms.inka.de': {
    'basedn':'o=Michaels',
    'binddn':r'mail=%(Email)s,o=%(O)s',
    'binddnsearch':r'(|(cn=*%s*)(mail=%s*)(initials=%s)(uid=%s))',
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel': {
        'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    'hiddenattrs':['userpassword','modifytimestamp','modifiersname'],
    'search_tablistattrs':['cn','mail','homephone','telephonenumber'],
    'search_tdtemplate': {
      'organizationalUnit':r'Org. Unit &quot;%(ou)s&quot; Tel.: %(telephonenumber)s',
      'rfc822MailGroup':r'Mailing list %(cn)s &lt;%(mail)s&gt;, see %(labeledurl)s',
      'account':r'User account <STRONG>%(uid)s<STRONG>',
    },
    'print_template':r'<FONT SIZE=-3>%(cn)s<BR>%(streetaddress)s<BR>%(l)s<BR>%(homephone)s<BR>%(telephonenumber)s<BR>%(mobile)s<BR>%(mail)s</FONT>',
    'print_cols':4,
    'dit': {
      # 'DN':(
      #   'DN component attribute of subordinate entries',
      #   [list of objectClasses],
      #   immutable
      # )
      'o=Michaels':('ou',['organizationalUnit'],0),
      'ou=Bizness,o=Michaels':('cn',['person','organizationalPerson','inetOrgPerson'],0),
      'ou=Friends,o=Michaels':('cn',['person','organizationalPerson','inetOrgPerson'],0),
      'ou=Musicians,o=Michaels':('cn',['person','organizationalPerson','inetOrgPerson','musician'],0),
      'ou=User,o=Michaels':('uid',['account','posixAccount'],0),
      'ou=Subscribed mailing lists,o=Michaels':('cn',['rfc822MailGroup'],0),
    },
    'read_template': {
      # 'object class':'pathname of HTML template file'
      'inetOrgPerson':'/home/michael/Proj/python/ldap/web2ldap/templates/read_inetOrgPerson.html',
      'posixAccount':'/home/michael/Proj/python/ldap/web2ldap/templates/read_posixAccount.html',
      'bankArrangement':'/home/michael/Proj/python/ldap/web2ldap/templates/read_bankArrangement.html',
    }
  },
  
  # To all hacker wanna-be's:
  # This as well is not a public accessible LDAP host. Don't try.
  # Adjust the settings to reflect your local LDAP installation.
  'ldap': {
    'basedn':'o=STP',
    'binddn':r'mail=%(Email)s,ou=Mitarbeiter,o=%(O)',
    'binddnsearch':r'(|(uid=%s)(cn=*%s*)(mail=%s*)(initials=%s))',
    'passwd_hashtypes':['crypt','sSha'],
    'search_maxhits':0,
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    'hiddenattrs':['userpassword','modifiersname'],
    'search_tablistattrs':['uid','cn','mail','homephone','telephonenumber'],
    'search_tdtemplate': {
      'organizationalUnit':r'Org. Unit &quot;%(ou)s&quot; Tel.: %(telephonenumber)s',
      'rfc822MailGroup':r'Mailing list %(cn)s &lt;%(mail)s&gt;, see %(labeledurl)s',
      'account':r'User account <STRONG>%(uid)s<STRONG>',
    },
    'print_template':r'<FONT SIZE=-3>%(cn)s<BR>%(telephonenumber)s<BR>Priv.:%(homephone)s<BR>Mobil:%(mobile)s<BR>%(mail)s</FONT>',
    'print_cols':4,
    'dit': {
      # 'DN':('DN component attribute of subordinate entries',[list of objectClasses],immutable)
      'o=STP':('ou',['organizationalUnit','groupOfNames'],0),
      'ou=Mailinglisten,o=STP':('mail',['top','rfc822MailGroup'],1),
      'ou=Mitarbeiter,o=STP':('uid',['top','person','organizationalPerson','inetOrgPerson'],0),
      'ou=Partner,o=STP':('cn',['top','person','organizationalPerson','inetOrgPerson'],0),
      'ou=Kunden,o=STP':('cn',['top','person','organizationalPerson','inetOrgPerson'],0)
    },
    'addform_template': {
      # 'object class':'pathname of HTML template file'
#      'inetOrgPerson':'../html/addform_inetOrgPerson.html',
#      'posixAccount':'../html/addform_posixAccount.html'
    },
    'read_template': {
      # 'object class':'pathname of HTML template file'
      'inetOrgPerson':'/usr/lib/web2ldap/html/read_inetOrgPerson.html',
      'posixAccount':'/usr/lib/web2ldap/html/read_posixAccount.html'
    }
  }

}

