##############################################################################
# ldapbase.py Version 0.1.4
# (c) by Michael Stroeder, michael.stroeder@propack-data.de
##############################################################################

import sys, string, re, ldap

ldap_mod_str = {ldap.MOD_ADD:'add',ldap.MOD_REPLACE:'replace',ldap.MOD_DELETE:'delete'}
ldap_searchscope_str = ['base','one','sub']
ldap_searchscope = {
  'base':ldap.SCOPE_BASE,
  'one':ldap.SCOPE_ONELEVEL,
  'sub':ldap.SCOPE_SUBTREE
}

dn_pattern   = r'([\w;.]+[\s]*=[^,]+)(,[ ]*[\w;.]+[\s]*=[^,]+)*'
dn_regex   = re.compile(dn_pattern)

# returns 1 if s is a LDAP DN
def is_dn(s):
  rm = dn_regex.match(s)
  return rm!=None

def normalize_dn(dn):
  result = string.split(dn,',')
  result = map(string.strip,result)
  return string.join(result,',')

# returns parent-DN of dn
def ParentDN(dn):
  return string.join(string.split(dn,',')[1:],',')

# returns parent-DN of dn
def SplitRDN(dn):
  try:
    rdn,basedn = string.split(dn,',',1)
    return (rdn,basedn)
  except ValueError:
    return (dn,'')

# returns a list of parent-DNs of dn
def ParentDNList(dn):
  result = []
  DNComponentList = string.split(dn,',')
  for i in range(1,len(DNComponentList)):
    result.append(string.join(DNComponentList[i:],','))
  return result

# parse a LDAP URL and return (host,dn,attributes,scope,filter)
# host         LDAP host
# dn           distinguished name
# attributes   list with attributes
# scope        search scope string
# filter       LDAP search filter
def parse_ldap_url(ldap_url):
  dummy,rest = string.split(ldap_url,'://',1)
  try:
    host,rest = string.split(rest,'/',1)
  except ValueError:
    host='' ; dn=rest
  paramlist=string.split(rest,'?')
  dn          = paramlist[0]
  try:
    attributes  = string.split(paramlist[1],',')
  except IndexError:
    attributes  = []
  try:
    scope       = paramlist[2]
  except IndexError:
    scope       = ''
  try:
    filter      = paramlist[3]
  except IndexError:
    filter      = ''
  return (host,dn,attributes,scope,filter)

