########################################################################
# web2ldap
# (c) by Michael Stroeder, michael.stroeder@inka.de
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import sys, string, re, \
       cgiforms, ldap, w2lcore, web2ldapcnf
from ldapbase import ParentDN#,ParentDNList,parse_ldap_url,is_dn,normalize_dn,dn_pattern
from msbase import union,listremove#,intersection

########################################################################
# Input form for add and modify
########################################################################

def w2l_AddModifyForm(outf,command,form):

  def PrintObjectClassForm(outf,form,command,ldap_dn,ldap_binddn,ldap_bindpasswd,ldap_host,ldap_rdn):

    outf.write('<DIV ID=InputDiv>')

    if command=='addform':
      outf.write('<H1>Add new entry</H1>\nChoose object class(es) for new entry.')

    elif command=='modifyform':
      outf.write('<H1>Modify entry</H1>\nYou may change the object class(es) for the entry.')

    if w2lcore.utf2display(form.accept_charset,ldap_rdn):
      ldap_rdn_input = '<INPUT TYPE=HIDDEN NAME="ldap_rdn" VALUE="%s">' % w2lcore.utf2display(form.accept_charset,ldap_rdn)
    else:
      ldap_rdn_input = ''

    outf.write("""
    <FORM ACTION="%s/%s" METHOD="%s" ENCTYPE="application/x-www-form-urlencoded" ACCEPT-CHARSET="%s">
    <INPUT TYPE=HIDDEN NAME="ldap_dn"     VALUE="%s">
    %s
    <INPUT TYPE=HIDDEN NAME="ldap_binddn"     VALUE="%s">
    <INPUT TYPE=HIDDEN NAME="ldap_bindpasswd" VALUE="%s">
    <INPUT TYPE=HIDDEN NAME="ldap_host"       VALUE="%s">
    <INPUT TYPE="submit" VALUE="Input form">
    <P>%s</P>
    </FORM></DIV>
    """ % ( \
      form.script_name,command,
      form.request_method,form.accept_charset,
      w2lcore.utf2display(form.accept_charset,ldap_dn),
      ldap_rdn_input,
      w2lcore.utf2display(form.accept_charset,ldap_binddn),
      w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
      ldap_host,
      form.field['ldap_oc'][0].inputfield()
    ))

  def PrintInputTable(outf,form,attrs,tableheader='',entry={}):
    outf.write('<TR><TH COLSPAN=2>%s</TH></TR>' % tableheader)
    for attr in attrs:
      datalist = entry.get(string.lower(attr),entry.get(attr,['']))
      for data in datalist:
	outf.write("""<TR>
      <TD>%s<INPUT TYPE=HIDDEN NAME="input_attr" VALUE="%s"></TD>
      <TD><INPUT NAME="input_data" VALUE="%s" SIZE=40></TD>
    </TR>
  """ % (w2lcore.AttrNameStr(attr),attr,w2lcore.utf2display(form.accept_charset,data)))

  from ldapoc import ldap_oc
  known_objectclasses = ldap_oc.keys()
  known_objectclasses.sort()

  if command=='addform':
    form.add(cgiforms.formInputClass('ldap_rdn','RDN of new entry',255,'.*'))

  form.add(cgiforms.formSelectClass('ldap_oc','Object classes',known_objectclasses,size=12,multiple=1))
  # FIX ME!!! This UGLY UGLY UGLY UGLY UGLY!
  form.field['ldap_oc'][0].regex = re.compile(r'[\w]*')

  ldap_session = w2lcore.DoParams(command,form)
  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn = ldap_session

  entry ={}

  if command=='addform':
    if 'ldap_rdn' in form.inputkeys:
      ldap_rdn = w2lcore.input2utf(form.accept_charset,form.field['ldap_rdn'][0].content)
    else:
      ldap_rdn = ''
    new_rdn_attr,objectclasses,dit_immutable = ldap_defaults.get('dit',{}).get(ldap_dn,('',[],0))
    form.field['ldap_oc'][0].setdefault(objectclasses)
  if command=='modifyform':
    ldap_rdn = ''
    if not ldap_dn:
      raise w2lcore.ErrorExitClass(form,ldap_session,'The DN of the entry to be modified is needed.')
    new_rdn_attr,objectclasses,dit_immutable = ldap_defaults.get('dit',{}).get(ParentDN(ldap_dn),('',[],0))

  if not dit_immutable:
    if not 'ldap_oc' in form.inputkeys:
      # Print form for selecting object class(es)
      if command=='modifyform':
	# Erst mal nur alle Attribute des Eintrags lesen
	try:
	  result_dnlist = ldap_obj.search_s(ldap_dn,ldap.SCOPE_BASE,"objectclass=*",['objectclass'],0)
	  entry = result_dnlist[0][1]
	  existing_objectclasses = entry.get('objectclass',entry.get('objectClass',[]))
	except ldap.NO_SUCH_OBJECT:
	  raise w2lcore.ErrorExitClass(form,ldap_session,'No entry <STRONG>%s</STRONG>.' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
	except:
	  exc_obj,exc_value,exc_traceback = sys.exc_info()
          raise w2lcore.ErrorExitClass(form,ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))
        # FIX ME! This is dirty!!!
        form.field['ldap_oc'][0].options = union(existing_objectclasses,form.field['ldap_oc'][0].options)
	form.field['ldap_oc'][0].options.sort()
	form.field['ldap_oc'][0].setdefault(existing_objectclasses)

      w2lcore.PrintHeader(outf,'',form.accept_charset)
      w2lcore.MainMenuTable(outf,form,ldap_session)
      PrintObjectClassForm(outf,form,command,w2lcore.utf2display(form.accept_charset,ldap_dn),w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_rdn))
      w2lcore.PrintFooter(outf)
      return

    else:
      # Read objectclass(es) from input form
      objectclasses = form.field['ldap_oc'][0].content

  if command=='modifyform':

    # Erst mal nur alle Attribute des Eintrags lesen
    try:
      # Get all attribute names of entries
      existing_attrs = ldap_obj.search_s(ldap_dn,ldap.SCOPE_BASE,"objectclass=*",None,web2ldapcnf.search_attrsonly)[0][1].keys()
      # Falls nicht nur ein Attributinhalt gelesen werden soll,
      # so werden alle Binaerattribute, fuer die ein
      # MIME-Typ-Mapping angegeben wurde, rausgestrichen
      read_attrs = existing_attrs[:]
      listremove(read_attrs,w2lcore.ldap_binaryattrkeys,1)
      # Read all non-binary attributes of entry
      result_dnlist = ldap_obj.search_s(ldap_dn,ldap.SCOPE_BASE,"objectclass=*",read_attrs,0)
      entry = result_dnlist[0][1]
    except ldap.NO_SUCH_OBJECT:
      raise w2lcore.ErrorExitClass(form,ldap_session,'No entry <STRONG>%s</STRONG>.' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
    except:
      exc_obj,exc_value,exc_traceback = sys.exc_info()
      raise w2lcore.ErrorExitClass(form,ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))

  ActionStr = '%s%s' % (string.upper(command[0]),command[1:-4])

  required_attrs = []
  allowed_attrs = []
  for oc in objectclasses:
    oc_def = ldap_oc.get(oc,{})
    required_attrs = union(required_attrs,oc_def.get('requires',[]))
    allowed_attrs = union(allowed_attrs,oc_def.get('allows',[]))

  donottouch_attrs = ['objectclass']+ldap_defaults.get('hiddenattrs',[])
  listremove(required_attrs,donottouch_attrs,1)
  listremove(required_attrs,w2lcore.ldap_binaryattrkeys,1)
  listremove(allowed_attrs,donottouch_attrs,1)
  listremove(allowed_attrs,w2lcore.ldap_binaryattrkeys,1)
  listremove(allowed_attrs,required_attrs,1)

  w2lcore.PrintHeader(outf,'',form.accept_charset)
  w2lcore.MainMenuTable(outf,form,ldap_session)
  outf.write("""<DIV ID=InputDiv>
<H1>%s entry</H1>
<FORM ACTION="%s/%s" METHOD="%s" ENCTYPE="application/x-www-form-urlencoded" ACCEPT-CHARSET="%s">
""" % (ActionStr,form.script_name,command[:-4],form.request_method,form.accept_charset))

  if command=='addform':
    if ldap_rdn or (not new_rdn_attr or not dit_immutable):
      outf.write('<P>RDN for new entry below <STRONG>%s</STRONG>:<BR><INPUT NAME="ldap_rdn" VALUE="%s" SIZE=50></P>' % (w2lcore.utf2display(form.accept_charset,ldap_dn),w2lcore.utf2display(form.accept_charset,ldap_rdn)))
  outf.write('<INPUT TYPE=HIDDEN NAME="ldap_dn" SIZE=50 VALUE="%s">' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))

  # Print input form according to object class(es)
  outf.write("""
  <INPUT TYPE=HIDDEN NAME="ldap_binddn"     VALUE="%s">
  <INPUT TYPE=HIDDEN NAME="ldap_bindpasswd" VALUE="%s">
  <INPUT TYPE=HIDDEN NAME="ldap_host"       VALUE="%s">
  <INPUT TYPE="submit" VALUE="%s"> <INPUT TYPE="reset" VALUE="Reset">
<TABLE ID=InputTable BORDER RULES=rows CELLSPACING="2%%" CELLPADDING="2%%">
<TR>
  <TD>%s</TD>
  <TD>%s</TD>
</TR>
""" % ( \
    w2lcore.utf2display(form.accept_charset,ldap_binddn),
    w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
    ldap_host,
    ActionStr,
    w2lcore.AttrNameStr('objectclass'),
    string.join(objectclasses,'<BR>')
  ))

  # Print chosen objectclass(es) as hidden input
  for oc in objectclasses:
    outf.write('<INPUT TYPE=HIDDEN NAME="ldap_oc" VALUE="%s">' % (oc))

  PrintInputTable(outf,form,required_attrs,'Required attributes',entry)
  PrintInputTable(outf,form,allowed_attrs,'Allowed attributes',entry)
  if command=='modifyform':
    # Print attributes which does not appear in the schema but
    # do exist in the entry
    listremove(read_attrs,['objectclass','modifytimestamp','creatorsname','createtimestamp','modifiersname','userpassword'],1)
    listremove(read_attrs,required_attrs,1)
    listremove(read_attrs,allowed_attrs,1)
    if read_attrs:
      PrintInputTable(outf,form,read_attrs,'Existing attributes not matching schema',entry)

  outf.write("""
<TR>
  <TD ALIGN=LEFT VALIGN=TOP>
    <P><EM>Raw LDIF data</EM></P>
    <FONT SIZE="-1">
      Notes:<BR>
      <UL>
        <LI>Lines containing "dn:" will be ignored</LI>
	<LI>Only the first entry (until first empty line) will be accepted</LI>
	<LI>NON-ASCII characters are not translated</LI>
        <LI>maximum length is set to %d bytes
      </UL>
    </FONT>
  </TD>
  <TD>
    <FONT SIZE="-1">
      <TEXTAREA NAME="input_ldif" VALUE="" WRAP=OFF ROWS=10 COLS=70></TEXTAREA>
    </FONT>
  </TD>
</TR>
</TABLE>
</FORM>
</DIV>
""" % (web2ldapcnf.ldif_maxbytes))

  w2lcore.PrintFooter(outf)

