########################################################################
# web2ldap
# (c) by Michael Stroeder, michael.stroeder@inka.de
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

Version = '0.7.5'

import sys, string, types, time, re, \
       cgiforms, ldap, cgissl, charset, web2ldapcnf

from ldapbase import ParentDN,ParentDNList,parse_ldap_url,is_dn,normalize_dn,dn_pattern

def utf2display(dispcharset,data):
  if type(data) is types.StringType:
    return charset.escapeHTML(charset.recode(data,'utf-8',dispcharset),'<>"(){}')
  elif type(data) is types.ListType:
    l = []
    for i in data:
      l.append(utf2display(dispcharset,i))
    return l
  else:
    raise TypeError, "expected string or list"

def input2utf(inputcharset,s):
  return charset.recode(s,inputcharset,'utf-8')

########################################################################
# Initialize some constants used throughout the whole script
########################################################################

host_pattern = '([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|[a-zA-Z]+[a-zA-Z0-9-]*(\.[a-zA-Z]+[a-zA-Z0-9-]*)*)?(:[0-9]*|)?'
url_pattern  = r'^(ftp|http|https|news|ldap|ldaps)?:(|//)?[^ ]*'
url_regex  = re.compile(url_pattern)
labeleduri_regex = re.compile(url_pattern+r' .*')
timestamp_pattern = r'^([0-9]){12,14}Z$'
timestamp_regex  = re.compile(timestamp_pattern)
mail_pattern = r'^([\w@.=/_ -]+)@([\w-]+)(\.[\w-]+)*$'
mail_regex = re.compile(mail_pattern)
ldap_binaryattrkeys = web2ldapcnf.ldap_binaryattr.keys()

search_attrkeys = web2ldapcnf.search_attr.keys()
search_attrkeys.sort()

# Reverse Mapping von search_attr
ldap_knownattr = {}
for i in search_attrkeys:
  ldap_knownattr[web2ldapcnf.search_attr[i]] = i
for i in ldap_binaryattrkeys:
  ldap_knownattr[i] = web2ldapcnf.ldap_binaryattr[i][0]
ldap_knownattrkeys = ldap_knownattr.keys()
ldap_knownattrkeys.sort()

search_option = {
  r'(%s=*%s*)':'contains',
  r'(!(%s=*%s*))':"doesn't contain",
  r'(%s=%s)':'is',
  r'(!(%s=%s))':'is not',
  r'(%s=%s*)':'begins with',
  r'(%s=*%s)':'ends with',
  r'(%s~=%s)':'sounds like'
}

class ErrorExitClass:

  def __init__(self,form,ldap_session,Msg):
    self.form = form
    self.ldap_session = ldap_session
    self.Msg = Msg

########################################################################
# haeufig gebrauchte Funktionen
########################################################################

def CommandButton(form,command,submitstr,binddn='',bindpasswd='',host='',dn='',ldapurl='',extrastr='',target='_top'):
  if ldapurl:
    dnstr = '<INPUT TYPE=HIDDEN NAME="ldap_url" VALUE="%s">' % (ldapurl)
  elif dn:
    dnstr = '<INPUT TYPE=HIDDEN NAME="ldap_dn" VALUE="%s">' % (dn)
  else:
    dnstr = ''
  return """
<FORM
  ID=CommandForm
  ACTION="%s/%s"
  METHOD="%s"
  TARGET="%s"
  ENCTYPE="application/x-www-form-urlencoded"
  ACCEPT-CHARSET="%s"
>
<INPUT TYPE=HIDDEN NAME="ldap_binddn"     VALUE="%s">
<INPUT TYPE=HIDDEN NAME="ldap_bindpasswd" VALUE="%s">
<INPUT TYPE=HIDDEN NAME="ldap_host"       VALUE="%s">
%s
<INPUT ID=CommandButton TYPE="submit" VALUE="%s">
%s
</FORM>""" % (
      form.script_name,
      command,
      web2ldapcnf.script_method,
      target,
      form.accept_charset,
      binddn,
      bindpasswd,
      host,
      dnstr,
      submitstr,
      extrastr
    )


# Print a table containing CommandButton()s
def CommandTable(outf,commandlist,div_id='CommandDiv',table_id='CommandTable'):
  outf.write('<DIV ID="%s"><TABLE ID="%s" CELLPADDING="0%%" CELLSPACING="0%%"><TR>' % (div_id,table_id))
  for s in commandlist:
    outf.write('<TD ID=MenuItem ALIGN=MIDDLE VALIGN=TOP>%s</TD>' % s)
  outf.write('</TR></TABLE></DIV>')


# Print the main menu at top of screen
def MainMenuTable(outf,form,ldap_session,current_dn=''):

  CommandList = []

  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn = ldap_session
  if current_dn:
    ldap_dn = current_dn

  if ldap_host:

    CommandList.append(CommandButton(form,'search','Go down',utf2display(form.accept_charset,ldap_binddn),utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,utf2display(form.accept_charset,ldap_dn),extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="one"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="(objectclass=*)">'))

    ldap_basedn = ldap_defaults.get('basedn','')
    if ldap_dn!=ldap_basedn:
      parent_dn_list = string.split(utf2display(form.accept_charset,ldap_dn),',')
      parent_dn_len = len(parent_dn_list)-1
      parent_dn_select = []
      i = 0
      dn = ldap_dn
      while (i<parent_dn_len) and (dn!=ldap_basedn):
	i=i+1
	dn = string.join(parent_dn_list[i:],',')
	parent_dn_select.append('<OPTION VALUE="%s">%s</OPTION>' % (dn,parent_dn_list[i]))
      if not ldap_basedn:
	parent_dn_select.append('<OPTION VALUE="">Root DSE</OPTION>')

      CommandList.append(CommandButton(form,'search','Go up',utf2display(form.accept_charset,ldap_binddn),utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,'',extrastr='<BR><FONT SIZE=-2><SELECT NAME="ldap_dn">%s</SELECT></FONT><INPUT TYPE=HIDDEN NAME="search_scope" VALUE="one"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="(objectclass=*)">' % string.join(parent_dn_select,'\n')))

    if ldap_basedn:
      search_dn = ldap_basedn
    else:
      search_dn = ldap_dn
    CommandList.extend([
	CommandButton(form,'searchform','Easy Search',utf2display(form.accept_charset,ldap_binddn),utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,utf2display(form.accept_charset,search_dn)),
	CommandButton(form,'filterform','Expert search',utf2display(form.accept_charset,ldap_binddn),utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,utf2display(form.accept_charset,ldap_dn)),
      ]
    )

    if ldap_dn:
      CommandList.extend(
	[
          CommandButton(form,'read','Read',utf2display(form.accept_charset,ldap_binddn),utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,utf2display(form.accept_charset,ldap_dn)),
	  CommandButton(form,'addform','New entry',utf2display(form.accept_charset,ldap_binddn),utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,utf2display(form.accept_charset,ldap_dn)),
	]
      )

    if ldap_binddn:
      CommandList.append(
        CommandButton(form,'passwd','Password',utf2display(form.accept_charset,ldap_binddn),utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,utf2display(form.accept_charset,ldap_binddn))
      )

    CommandList.append(
      CommandButton(form,'secinfo','Security',utf2display(form.accept_charset,ldap_binddn),utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,utf2display(form.accept_charset,ldap_dn))
    )

  CommandList.append(CommandButton(form,'login','Login','','',ldap_host))

  outf.write('<DIV ID=MainMenuDiv>')
  CommandTable(outf,CommandList,table_id='MainMenuTable')
  outf.write('<P><STRONG>%s</STRONG> on <STRONG>%s</STRONG>' % (
    DataStr(form,ldap_session,ldap_dn)+"Root DSE"*(utf2display(form.accept_charset,ldap_dn)==''),
    ldap_host
  ))
  if ldap_binddn:
    outf.write(', bound as <STRONG>%s</STRONG>' % (utf2display(form.accept_charset,ldap_binddn)))
  outf.write('</P></DIV>')


# Return a binddn formed by a template and
# a subject DN of a client certificate
def CreateDefaultBindDN(binddn_template,clientcertdn):

  def Splitclientcertdn(DN):
    result = {}
    s = string.split(DN[1:],'/')
    for i in s:
      try:
	id,value = string.split(i,'=',1)
	result[id] = value
      except:
	pass
    return result

  clientcertdnlist = Splitclientcertdn(clientcertdn)
  try:
    return charset.iso2utf(charset.asn12iso(binddn_template % clientcertdnlist))
  except KeyError:
    return ''


# Return a pretty HTML-formatted string for describing the attribute
def AttrNameStr(attr):
  attr = string.lower(attr)
  if attr in ldap_knownattrkeys:
    return '<STRONG>%s</STRONG><BR><EM>(%s)</EM>' % (ldap_knownattr[attr],attr)
  else:
    return '<EM>%s</EM>' % attr


# Return a pretty HTML-formatted string of the attribute data
def DataStr(form,ldap_session,data,attr='',commandbutton=0):

  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn = ldap_session

  try:
    if ldap.is_ldap_url(data):
      return '<TABLE><TR><TD>%s</TD><TD><A HREF="%s" target="_ldapurl">%s</A></TD></TR></TABLE>' % (
	       CommandButton(form,'read','Read',utf2display(form.accept_charset,ldap_binddn),utf2display(form.accept_charset,ldap_bindpasswd),'','',ldapurl_html4=utf2display(form.accept_charset,data)),
	       charset.utf2iso(data),
	       utf2display(form.accept_charset,data)
	     )
  except TypeError:
    pass

  rm = mail_regex.match(data)
  if rm!=None:
    return '<A HREF="mailto:%s">%s</A>' % (data,data)

  if is_dn(data):
    href_str = '<A HREF="ldap://%s/%s" target="_ldapurl">%s</A>' % (
             ldap_host,
	     charset.utf2iso(data),
	     utf2display(form.accept_charset,data)
	   )
    if commandbutton:
      return '<TABLE><TR><TD>%s</TD><TD>%s</TD></TR></TABLE>' % (
	       CommandButton(form,'read','Read',utf2display(form.accept_charset,ldap_binddn),utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,utf2display(form.accept_charset,data)),
	       href_str
	     )
    else:
      return href_str

  rm = url_regex.search(data)
  if rm!=None:
    rm = labeleduri_regex.match(data)
    if rm!=None:
      url,label = string.split(data,' ',1)
      return '<A HREF="%s" target="_labeledurl">%s</A>' % (url,utf2display(form.accept_charset,label))
    else:
      return '<A HREF="%s" target="_labeledurl">%s</A>' % (data,data)

  rm = timestamp_regex.match(data)
  if rm!=None:
    if len(data)==15:
      return '%s-%s-%sT%s:%s:%s-0000' % (data[0:4],data[4:6],data[6:8],data[8:10],data[10:12],data[12:14])
    elif len(data)==13:
      # old format without century
      year = int(data[0:2])
      if year<=50:
        return '20%s-%s-%sT%s:%s:%s-0000' % (data[0:2],data[2:4],data[4:6],data[6:8],data[8:10],data[10:12])
      else:
        return '19%s-%s-%sT%s:%s:%s-0000' % (data[0:2],data[2:4],data[4:6],data[6:8],data[8:10],data[10:12])
    else:
      return data
  return string.join(string.split(utf2display(form.accept_charset,data),'$'),'<BR>')


########################################################################
# Funktionen fuer immer wiederkehrende HTML-Ausgaben
########################################################################

# Ausdrucken eines HTML-Kopfes mit Titelzeile
def PrintHeader(outf,TitleMsg,HTTP_charset=''):
  if HTTP_charset:
    outf.write('Content-type: text/html; charset=%s\n' % (HTTP_charset))
  else:
    outf.write('Content-type: text/html\n')
  if web2ldapcnf.sec_expire:
    sec_expirestr = time.strftime('%A, %d-%b-%Y %H:%M:%S GMT',time.gmtime(time.time()+web2ldapcnf.sec_expire))
    outf.write('expires: %s\n\n<HTML>' % (sec_expirestr))
  else:
    outf.write('pragma: no-cache\n\n<HTML>')
  outf.write("""<HEAD>
  <TITLE>web2ldap %s:</TITLE>
  %s
</HEAD>
%s""" % (
    TitleMsg,
    web2ldapcnf.html_head,
    web2ldapcnf.html_bodybegin
  ))
  return

# Ausdrucken eines HTML-Endes
def PrintFooter(outf):
  outf.write("""<DIV ID=AboutDiv>
  <P ALIGN=CENTER>
    <FONT SIZE=-2>
      Powered by
      <A HREF="http://www.web2ldap.de/" target="_web2ldap">web2ldap %s</A>
    </FONT>
  </P>
</DIV>
</BODY>
</HTML>""" % (Version))
  return

def DoParams(command,form):

  try:
    form.getparams(ignoreemptyparams=1)
  except cgiforms.formContentLengthException,e:
    raise ErrorExitClass(form,('','','',None,{},'',''),'Content length invalid. Received %d bytes (max. &d allowed).' % (e.contentlength,e.maxcontentlength))
  except cgiforms.formParamNameException,e:
    raise ErrorExitClass(form,('','','',None,{},'',''),'Unknown parameter <STRONG>%s</STRONG>.' % (e.name))
  except cgiforms.formParamContentException,e:
    raise ErrorExitClass(form,('','','',None,{},'',''),'Content of field <STRONG>%s</STRONG> has invalid format.<BR>Input was: &quot;%s&quot;' % (e.text,e.content))
  except cgiforms.formParamStructException,e:
    raise ErrorExitClass(form,('','','',None,{},'',''),'Too many (%d) parameters for field <STRONG>%s</STRONG>.' % (e.count,e.name))
  except cgiforms.formParamLengthException,e:
    raise ErrorExitClass(form,('','','',None,{},'',''),'Content too long. Field <STRONG>%s</STRONG> has %d characters but is limited to %d.' % (e.text,e.length,e.maxlength))

  if 'ldap_url' in form.inputkeys:
    ldap_url = input2utf(form.accept_charset,form.field['ldap_url'][0].content)
    if ldap.is_ldap_url(ldap_url):
      ldap_host,ldap_dn,search_attr,search_scope,search_filterstr = parse_ldap_url(ldap_url)
    else:
      raise ErrorExitClass(form,(ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn),'LDAP URL <STRONG>%s</STRONG> has invalid format.' % (ldap_url))

  else:

    ldap_url=''
    if 'ldap_host' in form.inputkeys:
      ldap_host = form.field['ldap_host'][0].content
    else:
      ldap_host = ''
    if 'ldap_dn' in form.inputkeys:
      ldap_dn = input2utf(form.accept_charset,form.field['ldap_dn'][0].content)
    else:
      ldap_dn = ''

  if not ldap_host:
    return ('','','',None,{},'','')

  ldap_defaults = web2ldapcnf.ldap_def.get(ldap_host,{})

  sec_valid_dn = ldap_defaults.get('sec_valid_dn','')
  sec_valid_idn = ldap_defaults.get('sec_valid_idn','')
  sec_ssllevel = cgissl.SecLevel(web2ldapcnf.sec_sslacceptedciphers,sec_valid_dn,sec_valid_idn)
  # Ist der Sicherheitslevel ausreichend fuer das geforderte Kommando?
  sec_reqlevel = ldap_defaults.get('sec_reqlevel',{}).get(command,web2ldapcnf.sec_reqlevel_default)
  if sec_ssllevel < sec_reqlevel:
    raise ErrorExitClass(form,(ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn),'Security level %d required! The security level of your access was only %d.' % (sec_reqlevel,sec_ssllevel))

  ldap_basedn = charset.iso2utf(ldap_defaults.get('basedn',''))

  try:
    ldap_obj = ldap.open(ldap_host)
  except:
    ldap_host = ''
    exc_obj,exc_value,exc_traceback = sys.exc_info()
    raise ErrorExitClass(form,(ldap_host,'','',None,{},'',''),'Unhandled exception during connecting to %s:<P>%s, %s</P>' % (ldap_host,exc_obj,exc_value))

  if 'ldap_bindpasswd' in form.inputkeys:

    ldap_bindpasswd = input2utf(form.accept_charset,form.field['ldap_bindpasswd'][0].content)

    if 'ldap_binddn' in form.inputkeys:

      ldap_binddn = input2utf(form.accept_charset,form.field['ldap_binddn'][0].content)

      if not is_dn(ldap_binddn):
	# Do a smart login by searching unique entry
	ldap_searchdnfilter = string.replace(ldap_defaults.get('binddnsearch',''),'%s',ldap_binddn)
	try:
	  result = ldap_obj.search_s(ldap_basedn,ldap.SCOPE_SUBTREE,ldap_searchdnfilter,['objectclass'],web2ldapcnf.search_attrsonly)
	except ldap.NO_SUCH_OBJECT:
	  raise ErrorExitClass(
	    form,
	    (ldap_host,'','',None,{},'',''),
	    'Smart login did not find any entry for user name <STRONG>%s</STRONG>.' % (ldap_binddn)
	  )
	except ldap.FILTER_ERROR:
	  raise ErrorExitClass(
	    form,
	    (ldap_host,'','',None,{},'',''),
	    'Bad search filter for smart login.<BR>This is likely a configuration problem.'
	  )
	except:
	  exc_obj,exc_value,exc_traceback = sys.exc_info()
	  raise ErrorExitClass(
	    form,
	    ('','','',None,{},'',''),
	    'Unhandled exception during smart login on %s:<P>%s, %s.</P>' % (ldap_host,exc_obj,exc_value)
	  )
	if result:
          if len(result)==1:
            ldap_binddn = result[0][0]
	  else:
    	    raise ErrorExitClass(form,
	      (ldap_host,ldap_binddn,ldap_bindpasswd,None,{},'',''),
	      'Smart login did not find a unique entry to authenticate for user name.<BR>It might help to provide a more detailed user name.'
	    )
	else:
	  raise ErrorExitClass(
	    form,
	    (ldap_host,'','',None,{},'',''),
	    'Smart login did not find any entry for user name.'
	  )
    else:
      ldap_binddn = CreateDefaultBindDN(ldap_defaults.get('binddn',''),form.env.get('SSL_CLIENT_DN',''))

    ldap_binddn = normalize_dn(ldap_binddn)

  else:
    ldap_binddn = ''
    ldap_bindpasswd = ''

  ldap_dn = normalize_dn(ldap_dn)

  try:
    ldap_obj.bind_s(ldap_binddn,ldap_bindpasswd,ldap.AUTH_SIMPLE)
  except ldap.INVALID_CREDENTIALS:
    raise ErrorExitClass(form,('','','',None,{},'',''),'Wrong password for binding as <STRONG>%s</STRONG> to %s.' % (utf2display(form.accept_charset,ldap_binddn),ldap_host))
  except ldap.INAPPROPRIATE_AUTH:
    raise ErrorExitClass(form,('','','',None,{},'',''),'Wrong password for binding as <STRONG>%s</STRONG> to %s.' % (utf2display(form.accept_charset,ldap_binddn),ldap_host))
  except:
    exc_obj,exc_value,exc_traceback = sys.exc_info()
    raise ErrorExitClass(form,(ldap_host,'','',None,{},'',''),'Unhandled exception during binding as <STRONG>%s</STRONG> to %s:<P>%s, %s.</P>' % (utf2display(form.accept_charset,ldap_binddn),ldap_host,exc_obj,exc_value))

  return (ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn)

