########################################################################
# web2ldap
# (c) by Michael Stroeder, michael.stroeder@inka.de
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import sys, string, re, \
       cgiforms, ldap, ldapbase, w2lcore, web2ldapcnf
from msbase import listremove

##############################################################################
# Modify existing entry
##############################################################################

def w2l_Modify(outf,command,form):

  def ModifyListTable(form,modlist):
    stringlist = []
    stringlist.append('<TABLE BORDER RULES=rows CELLSPACING=0 CELLPADDING=2%%>')
    for i in modlist:
      if i[0]==ldap.MOD_ADD:
	stringlist.append('<TR><TD>%s</TD><TD>%s</TD><TD>%s</TD></TR>' % (ldapbase.ldap_mod_str[i[0]],w2lcore.AttrNameStr(i[1]),i[2]))
      else:
	stringlist.append('<TR><TD>%s</TD><TD>%s</TD><TD>%s</TD></TR>' % (ldapbase.ldap_mod_str[i[0]],w2lcore.AttrNameStr(i[1]),string.join(w2lcore.utf2display(form.accept_charset,i[2]),'<BR>')))
    stringlist.append('</TABLE>')
    return string.join(stringlist,'\n')

  from ldapoc import ldap_oc
  known_objectclasses = ldap_oc.keys()
  form.add(cgiforms.formSelectClass('ldap_oc','Object classes',known_objectclasses,size=10,multiple=1))
  # FIX ME!!! This UGLY UGLY UGLY UGLY UGLY!
  form.field['ldap_oc'][0].regex = re.compile(r'[\w]*')

  form.add(cgiforms.formInputClass('input_attr','Attributename',255,'[a-zA-Z]+[a-zA-Z0-9;]*',multiple=web2ldapcnf.input_maxattrs))
  form.add(cgiforms.formInputClass('input_data','Attributedata',255,'.*',multiple=web2ldapcnf.input_maxattrs))
  form.add(cgiforms.formInputClass('input_ldif','LDIF data',web2ldapcnf.ldif_maxbytes,'.*'))

  ldap_session = w2lcore.DoParams(command,form)
  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn = ldap_session

  if not ldap_dn:
    raise w2lcore.ErrorExitClass(form,ldap_session,'Command <STRONG>%s</STRONG> requires DN.' % command)

  try:
    # Get all existing attribute names of entry
    existing_attrs = ldap_obj.search_s(ldap_dn,ldap.SCOPE_BASE,"objectclass=*",None,web2ldapcnf.search_attrsonly)[0][1].keys()
    listremove(existing_attrs,w2lcore.ldap_binaryattrkeys+ldap_defaults.get('hiddenattrs',[]),1)
    # Read all non-binary attribute data of entry
    result_dnlist = ldap_obj.search_s(ldap_dn,ldap.SCOPE_BASE,"objectclass=*",existing_attrs,0)
    # FIX ME?!?
    existing_attrs = map(string.lower,existing_attrs)
    old_entry = result_dnlist[0][1]

  except ldap.NO_SUCH_OBJECT:
    raise w2lcore.ErrorExitClass(form,ldap_session,'No entry <STRONG>%s</STRONG>.' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
  except:
    exc_obj,exc_value,exc_traceback = sys.exc_info()
    raise w2lcore.ErrorExitClass(form,ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))


  old_objectclasses = old_entry.get('objectclass',[])
  old_objectclasses.sort()

  modlist = []
  if 'ldap_oc' in form.inputkeys:
   # Object class(es) were defined in users input
    new_objectclasses = form.field['ldap_oc'][0].content
    new_objectclasses.sort()
    if map(string.lower,old_objectclasses)!=map(string.lower,new_objectclasses):
      modlist.append((ldap.MOD_REPLACE,'objectClass',new_objectclasses))

  if 'input_ldif' in form.inputkeys:
    from StringIO import StringIO
    import ldif
    f = StringIO(form.field['input_ldif'][0].content)
    ldif_data = ldif.ParseLDIF(f,maxentries=1)
    new_entry = ldif_data[0][1]
  else:
    new_entry = {}

  if 'input_attr' in form.inputkeys:
    for i in range(len(form.field['input_attr'][0].content)):
      attr = form.field['input_attr'][0].content[i]
      if attr:
	data = w2lcore.input2utf(form.accept_charset,form.field['input_data'][0].content[i])
	if new_entry.has_key(attr):
	  new_entry[attr].append(data)
	else:
	  new_entry[attr] = [data]

  # Modify-Liste zumsammenbauen
  for attr in new_entry.keys():
    if string.lower(attr) in existing_attrs:
      attr_lower = string.lower(attr)
      old_data = old_entry.get(attr,old_entry.get(attr_lower,[]))
      if new_entry[attr]!=old_data:
	if old_data and (new_entry[attr]==[]) or (new_entry[attr]==['']):
	  # delete an existing attribute
	  modlist.append((ldap.MOD_DELETE,attr,old_data))
	else:
	  # modify an existing attribute
	  modlist.append((ldap.MOD_REPLACE,attr,new_entry[attr]))
    else:
      for data in new_entry[attr]:
	if data:
	  # neue Attribute hinzufuegen
	  modlist.append((ldap.MOD_ADD,attr,data))

  if modlist:
    # Send modify-list to host
    try:
      ldap_obj.modify_s(ldap_dn,modlist)
    except ldap.NO_SUCH_OBJECT:
      raise w2lcore.ErrorExitClass(form,ldap_session,'No such object <STRONG>%s</STRONG>: The object to be modified does not exist. Wrong DN?' % w2lcore.utf2display(form.accept_charset,ldap_dn))
    except ldap.OBJECT_CLASS_VIOLATION:
      raise w2lcore.ErrorExitClass(form,ldap_session,'Object class violation: Maybe a required attribute is missing or some of the given attributes are not allowed.')
    except ldap.INSUFFICIENT_ACCESS:
      raise w2lcore.ErrorExitClass(form,ldap_session,'You are not allowed to modify entry <STRONG>%s</STRONG>.' % w2lcore.utf2display(form.accept_charset,ldap_dn))
    except:
      exc_obj,exc_value,exc_traceback = sys.exc_info()
      raise w2lcore.ErrorExitClass(form,ldap_session,'<P>Unhandled exception: %s, %s.</P>Input data:<BR>%s' % (exc_obj,exc_value,ModifyListTable(form,modlist)))

  w2lcore.PrintHeader(outf,'',form.accept_charset)
  w2lcore.MainMenuTable(outf,form,ldap_session)
  w2lcore.CommandTable(outf,[
    w2lcore.CommandButton(form,'vcard','vCard',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),target='_labeledurl'),
    w2lcore.CommandButton(form,'ldif','LDIF',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),
	          extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="base"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="(objectClass=*)">',target='_altformat'),
    w2lcore.CommandButton(form,'dsml','DSML',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),
	          extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="base"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="(objectClass=*)">',target='_altformat'),
    w2lcore.CommandButton(form,'modifyform','Modify',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn)),
    w2lcore.CommandButton(form,'modrdn','Modify RDN',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn)),
    w2lcore.CommandButton(form,'delete','Delete',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn)),
    w2lcore.CommandButton(form,'passwd','Password of entry',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn))
  ],div_id='ContextMenuDiv',table_id='ContextMenuTable')
  outf.write('<DIV ID=MessageDiv>')
  if modlist:
    outf.write('<P>Modified entry <STRONG>%s</STRONG> with your input data:</P>\n%s' % (w2lcore.utf2display(form.accept_charset,ldap_dn),ModifyListTable(form,modlist)))
  else:
    outf.write('No attributes modified.')
  outf.write('</DIV>')
  w2lcore.PrintFooter(outf)


