########################################################################
# web2ldap
# (c) by Michael Stroeder, michael.stroeder@inka.de
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import sys,string,charset,web2ldapcnf,cgiforms,ldap,w2lcore

########################################################################
# Change userpassword
########################################################################

def w2l_Passwd(outf,command,form):

  form.add(cgiforms.formFieldClass('passwd_newpasswd','New password',100,'.*',multiple=2))

  # Try to determine the avaiable hash types
  local_hashtypes = [('','None')]

  # Check if all required modules are in place to
  # determine the available encryption methods
  try:
    import whrandom
    import base64
    import crypt
    local_hashtypes.append(('crypt','Crypt'))
    import md5
    local_hashtypes.append(('md5','MD5'))
    local_hashtypes.append(('smd5','salted MD5'))
    import sha
    local_hashtypes.append(('sha','SHA-1'))
    local_hashtypes.append(('ssha','salted SHA-1'))
  except ImportError:
    pass

  form.add(cgiforms.formSelectClass('passwd_hash','Hash',local_hashtypes,local_hashtypes[-1]))

  ldap_session = w2lcore.DoParams(command,form)
  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn = ldap_session

  if not ldap_binddn:
    ldap_binddn = ldap_dn

  if ldap_dn and \
     ('passwd_newpasswd' in form.inputkeys):

    try:
      ldap_obj.bind_s(ldap_binddn,ldap_bindpasswd,ldap.AUTH_SIMPLE)
    except ldap.INVALID_CREDENTIALS:
      raise ErrorExitClass(form,ldap_session,'Wrong password for binding as <STRONG>%s</STRONG> to %s.' % (w2lcore.utf2display(form.accept_charset,ldap_binddn),ldap_host))
    except ldap.INAPPROPRIATE_AUTH:
      raise ErrorExitClass(form,ldap_session,'Wrong password for binding as <STRONG>%s</STRONG> to %s.' % (w2lcore.utf2display(form.accept_charset,ldap_binddn),ldap_host))
    except:
      exc_obj,exc_value,exc_traceback = sys.exc_info()
      raise ErrorExitClass(form,ldap_session,'<P>Unhandled exception during binding as <STRONG>%s</STRONG> to %s:</P>%s, %s.' % (w2lcore.utf2display(form.accept_charset,ldap_binddn),ldap_host,exc_obj,exc_value))

    if len(form.field['passwd_newpasswd'][0].content)!=2:
      raise ErrorExitClass(form,ldap_session,'Repeat password!')

    if form.field['passwd_newpasswd'][0].content[0]!=form.field['passwd_newpasswd'][0].content[1]:
      raise ErrorExitClass(form,ldap_session,'New passwords do not match!')

    if not form.field['passwd_newpasswd'][0].content[0]:
      raise ErrorExitClass(form,ldap_session,'Empty passwords not allowed!')

    passwdinput = form.field['passwd_newpasswd'][0].content[0]

    if 'passwd_hash' in form.inputkeys:
      passwd_hash = string.lower(form.field['passwd_hash'][0].content)
      if passwd_hash in ['crypt','smd5','ssha']:
        crypt_salt = ''
        for i in range(2):
  	  crypt_salt = crypt_salt+charset.crypt_alphabet[whrandom.randint(0,charset.crypt_alphabet_len-1)]
      else:
	crypt_salt = ''
      if passwd_hash=='crypt':
	new_passwd = '{crypt}%s' % crypt.crypt(form.field['passwd_newpasswd'][0].content[0],crypt_salt)
      elif passwd_hash in ['md5','smd5']:
	new_passwd = string.strip('{%s}%s' % (passwd_hash,base64.encodestring(md5.new(passwdinput+crypt_salt).digest()+crypt_salt)))
      elif passwd_hash in ['sha','ssha']:
	new_passwd = string.strip('{%s}%s' % (passwd_hash,base64.encodestring(sha.new(passwdinput+crypt_salt).digest()+crypt_salt)))
    else:
      new_passwd = passwdinput

    modlist = [(ldap.MOD_REPLACE,'userPassword',new_passwd)]

    # Passwort aendern
    try:
      ldap_obj.modify_s(ldap_dn,modlist)
    except ldap.NO_SUCH_OBJECT:
      raise ErrorExitClass(form,ldap_session,'No entry <STRONG>%s</STRONG>.' % w2lcore.utf2display(form.accept_charset,ldap_dn))
    except ldap.INSUFFICIENT_ACCESS:
      raise ErrorExitClass(form,ldap_session,'You are not allowed to modify entry <STRONG>%s</STRONG>.' % w2lcore.utf2display(form.accept_charset,ldap_dn))
    except:
      exc_obj,exc_value,exc_traceback = sys.exc_info()
      raise ErrorExitClass(form,ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))

    w2lcore.PrintHeader(outf,'Changed password',form.accept_charset)
    outf.write('<DIV ID=MessageDiv>')
    if ldap_dn==ldap_binddn:
      w2lcore.CommandTable(outf,[
	w2lcore.CommandButton(form,'login','Login as','','dummy',ldap_host,'',
	              extrastr='<INPUT TYPE=HIDDEN NAME="login_binddn" VALUE="%s">' % (w2lcore.utf2display(form.accept_charset,ldap_binddn)))
      ],div_id='ContextMenuDiv',table_id='ContextMenuTable')
      outf.write('<P>A new login is required!</P>')
    else:
      w2lcore.MainMenuTable(outf,form,ldap_session)
    outf.write('<P>Changed password of entry <STRONG>%s</STRONG>.</P></DIV>' % w2lcore.utf2display(form.accept_charset,ldap_dn))
    w2lcore.PrintFooter(outf)

  else:

    if ldap_defaults.has_key('passwd_hashtypes'):
      # The set of hash types are restricted by local configuration
      config_hashtypes = map(string.lower,ldap_defaults['passwd_hashtypes'])
      default_hashtypes = []
      for hash_type in local_hashtypes:
	if hash_type[0] in config_hashtypes:
          default_hashtypes.append(hash_type)
      form.field['passwd_hash'][0].options = default_hashtypes

    # Leeres Passworteingabeformular ausgeben
    w2lcore.PrintHeader(outf,'Change password',form.accept_charset)
    w2lcore.MainMenuTable(outf,form,ldap_session)
    outf.write("""<DIV ID=InputDiv>
<H1>Change password</H1>
<FORM ACTION="%s/passwd" METHOD="%s" ENCTYPE="application/x-www-form-urlencoded" ACCEPT-CHARSET="%s">
  <TABLE CELLPADDING="5%%" CELLSPACING=0>
  <TR>
    <TD>Change password for</TD>
    <TD><INPUT NAME="ldap_dn" MAXLENGTH=200 SIZE=30 VALUE="%s"> on %s</TD>
    <INPUT TYPE=HIDDEN NAME="ldap_host" VALUE="%s"></TD>
    <INPUT TYPE=HIDDEN NAME="ldap_binddn" VALUE="%s"></TD>
  </TR>
  <TR>
    <TD>Password for <STRONG>%s</STRONG></TD>
    <TD><INPUT TYPE="PASSWORD" NAME="ldap_bindpasswd" MAXLENGTH=30 SIZE=15 VALUE=""></TD>
  </TR>
  <TR>
  <TR>
    <TD>New password</TD>
    <TD><INPUT TYPE="PASSWORD" NAME="passwd_newpasswd" MAXLENGTH=30 SIZE=30 VALUE=""></TD>
  </TR>
  <TR>
    <TD>repeat new password</TD>
    <TD><INPUT TYPE="PASSWORD" NAME="passwd_newpasswd" MAXLENGTH=30 SIZE=30 VALUE=""></TD>
  </TR>
  <TR>
    <TD>Password encryption method</TD>
    <TD>%s</TD>
  </TR>
</TABLE>
<INPUT TYPE="submit" VALUE="Change Password">
<INPUT TYPE="reset" VALUE="Reset">
</FORM>
</DIV>
""" % (form.script_name,web2ldapcnf.script_method,form.accept_charset,w2lcore.utf2display(form.accept_charset,ldap_dn),ldap_host,ldap_host,w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_binddn),form.field['passwd_hash'][0].inputfield()))

    w2lcore.PrintFooter(outf)


