########################################################################
# web2ldap
# (c) by Michael Stroeder, michael.stroeder@inka.de
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import sys, string, \
       msbase, cgiforms, ldap, w2lcore, web2ldapcnf
from msbase import listremove,intersection,union

##############################################################################
# Read existing entry and output as HTML od vCard
##############################################################################

def w2l_Read(outf,command,form):

  form.add(cgiforms.formInputClass('read_attr','Read attribute',255,'[a-zA-Z]+[a-zA-Z0-9;]*'))
  form.add(cgiforms.formInputClass('read_attrindex','Read attribute',255,'[0-9]+'))

  ldap_session = w2lcore.DoParams(command,form)
  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn = ldap_session

  if not ldap_dn:
    raise w2lcore.ErrorExitClass(form,ldap_session,'Command <STRONG>%s</STRONG> requires DN.' % command)

  # Erst mal nur alle Attribute des Eintrags lesen
  try:
    result_dnlist = ldap_obj.search_s(ldap_dn,ldap.SCOPE_BASE,"objectclass=*",None,web2ldapcnf.search_attrsonly)
    all_attrs = result_dnlist[0][1].keys()

  except ldap.NO_SUCH_OBJECT:
    raise w2lcore.ErrorExitClass(form,ldap_session,'No entry <STRONG>%s</STRONG>.' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
  except:
    exc_obj,exc_value,exc_traceback = sys.exc_info()
    raise w2lcore.ErrorExitClass(form,ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))

  ldap_hiddenattrs = ldap_defaults.get('hiddenattrs',[])
  # Attribute herausloeschen, welche unsichtbar bleiben sollen
  listremove(all_attrs,ldap_hiddenattrs,1)

  # Wurde eine Attributwunschliste angegeben?
  if 'read_attr' in form.inputkeys:
    wanted_attrs = [form.field['read_attr'][0].content]
  else:
    wanted_attrs = all_attrs

  # Falls nicht nur ein Attributinhalt gelesen werden soll,
  # so werden alle Binaerattribute, fuer die ein
  # MIME-Typ-Mapping angegeben wurde, rausgestrichen
  read_attrs = wanted_attrs[:]
  if len(read_attrs)>1:
    listremove(read_attrs,w2lcore.ldap_binaryattrkeys,1)

  # Lesen ausfhren
  try:
    result_dnlist = ldap_obj.search_s(ldap_dn,ldap.SCOPE_BASE,"objectclass=*",read_attrs,0)
  except ldap.NO_SUCH_OBJECT:
    raise w2lcore.ErrorExitClass(form,ldap_session,'Noch such object.')
  except:
    exc_obj,exc_value,exc_traceback = sys.exc_info()
    raise w2lcore.ErrorExitClass(form,ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))

  entry = result_dnlist[0][1]

  # Ausgabe der gelesenen Eintragsdaten

  if command=='read':

    if (len(read_attrs)==1) and \
       (string.lower(read_attrs[0]) in w2lcore.ldap_binaryattrkeys):
      # Ausgabe eines einzelnen Binaerdatums

      import cgihelper

      if 'read_attrindex' in form.inputkeys:
        read_attrindex = string.atoi(form.field['read_attrindex'][0].content)
	if read_attrindex >= len(entry[read_attrs[0]]):
          raise w2lcore.ErrorExitClass(form,ldap_session,'Illegal index for multi-valued binary attribute.')
      else:
        read_attrindex = 0

      browsertype,browserversion = cgihelper.BrowserType(form.env.get('HTTP_USER_AGENT',''))
      if web2ldapcnf.ldap_browsermimetypes.has_key(read_attrs[0]):
	mimetype = web2ldapcnf.ldap_browsermimetypes[read_attrs[0]].get(browsertype,web2ldapcnf.ldap_binaryattr[read_attrs[0]][1])
      else:
	mimetype = web2ldapcnf.ldap_binaryattr[string.lower(read_attrs[0])][1]
      outf.write('Content-type: %s\nPragma: no-cache\n\n' % mimetype)
      outf.write(entry[read_attrs[0]][read_attrindex])

    else:

      nice_dn = w2lcore.utf2display(form.accept_charset,ldap_dn)

      # Ausgabe des ganzen Eintrags mit mehreren Attributen
      w2lcore.PrintHeader(outf,nice_dn,form.accept_charset)
      w2lcore.MainMenuTable(outf,form,ldap_session)
      w2lcore.CommandTable(outf,[
	w2lcore.CommandButton(form,'vcard','vCard',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),target='_labeledurl'),
	w2lcore.CommandButton(form,'ldif','LDIF',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),
	              extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="base"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="(objectClass=*)">',target='_altformat'),
	w2lcore.CommandButton(form,'dsml','DSML',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),
	              extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="base"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="(objectClass=*)">',target='_altformat'),
	w2lcore.CommandButton(form,'modifyform','Modify',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn)),
	w2lcore.CommandButton(form,'modrdn','Modify RDN',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn)),
	w2lcore.CommandButton(form,'delete','Delete',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn)),
	w2lcore.CommandButton(form,'passwd','Password of entry',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn)),
	w2lcore.CommandButton(form,'login','Login as','','dummy',ldap_host,'',
	              extrastr='<INPUT TYPE=HIDDEN NAME="login_binddn" VALUE="%s">' % (w2lcore.utf2display(form.accept_charset,ldap_dn))),
      ],div_id='ContextMenuDiv',table_id='ContextMenuTable')

      objectclasses = entry.get('objectclass',entry.get('objectClass',[]))

      read_template_dict = ldap_defaults.get('read_template',{})
      template_oc = intersection(objectclasses,read_template_dict.keys())

      if template_oc:

	# Templates defined => display the entry with the help of the template
	displayentry = msbase.DefaultDict('&nbsp;')
	attr_list=entry.keys()
	for attr in attr_list:
          itemlist = []
          for item in entry[attr]:
            itemlist.append(w2lcore.DataStr(form,ldap_session,item))
	  displayentry[attr] = string.join(itemlist,', ')

	for oc in template_oc:
          template_str = open(read_template_dict[oc],'r').read()
	  try:
  	    template_str = open(read_template_dict[oc],'r').read()
  	    outf.write(template_str % displayentry)
	  except IOError:
	    outf.write('<DIV ID=MessageDiv><P>Template file for <STRONG>%s</STRONG> not found.</P></DIV>' % oc)

      else:

	wanted_attrs = map(string.lower,wanted_attrs)
	# No templates defined => display the entry in a raw way
	from ldapoc import ldap_oc
	required_attrs = []
	allowed_attrs = []
	for oc in objectclasses:
	  oc_def = ldap_oc.get(oc,{})
	  required_attrs = union(required_attrs,oc_def.get('requires',[]))
	  allowed_attrs = union(allowed_attrs,oc_def.get('allows',[]))
	required_attrs = intersection(map(string.lower,required_attrs),wanted_attrs)
	allowed_attrs = intersection(map(string.lower,allowed_attrs),wanted_attrs)
	nomatching_attrs = wanted_attrs[:]
	listremove(nomatching_attrs,required_attrs,1)
	listremove(nomatching_attrs,allowed_attrs,1)
	listremove(required_attrs,['objectclass'],1)
	listremove(allowed_attrs,['objectclass'],1)
	listremove(nomatching_attrs,['objectclass'],1)

	def PrintAttrRows(outf,form,ldap_session,entry,attrs,read_attrs,Comment=''):
          ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn = ldap_session
          if attrs:
	    display_attrs = []
	    for attr in read_attrs:
	      if string.lower(attr) in attrs:
		display_attrs.append(attr)
            attrs.sort()
            if Comment:
              outf.write('<TR><TH COLSPAN=2 ALIGN=LEFT><STRONG>%s<STRONG></TH></TR>\n' % (Comment))
	    for attr in display_attrs:
	      if not (string.lower(attr) in w2lcore.ldap_binaryattrkeys):
        	datalist = entry.get(attr,entry.get(string.lower(attr),[]))
                if datalist:
		  outf.write('<TR><TD WIDTH="30%%" ROWSPAN=%d>%s</TD>' % (len(datalist),w2lcore.AttrNameStr(attr)))
        	  # Ausgabe des Attribut-Inhalts
		  outf.write('<TD ALIGN=LEFT>%s</TD></TR>\n' % w2lcore.DataStr(form,ldap_session,datalist[0],commandbutton=1))
		  for data in datalist[1:]:
		    outf.write('<TR><TD ALIGN=LEFT>%s</TD></TR>\n' % w2lcore.DataStr(form,ldap_session,data,commandbutton=1))
                else:
		  outf.write('<TR><TD WIDTH="30%%">%s</TD><TD ALIGN=LEFT>&nbsp;</TD></TR>\n' % (w2lcore.AttrNameStr(attr)))
	      else:
		outf.write('<TR><TD WIDTH="30%%">%s</TD><TD ALIGN=LEFT>%s</TD></TR>\n' % (
		  w2lcore.AttrNameStr(attr),
		  w2lcore.CommandButton(
		    form,'read','View/Load',
		    w2lcore.utf2display(form.accept_charset,ldap_binddn),
		    w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
		    ldap_host,
		    w2lcore.utf2display(form.accept_charset,ldap_dn),
		    extrastr='<INPUT TYPE=HIDDEN NAME="read_attr" VALUE="%s"><INPUT TYPE=HIDDEN NAME="read_attrindex" VALUE="0">' % attr
		  )
		))

	# Eigentliche Ausgabe des gelesenen Eintrags
	outf.write('<TABLE ID=ReadTable WIDTH="100%%" BORDER RULES=rows NOSHADE CELLPADDING="2%%" CELLSPACING=0>')

	PrintAttrRows(outf,form,ldap_session,entry,['objectclass'],read_attrs,'Object classes')
	PrintAttrRows(outf,form,ldap_session,entry,required_attrs,all_attrs,'Required Attributes')
	PrintAttrRows(outf,form,ldap_session,entry,allowed_attrs,all_attrs,'Allowed Attributes')
	PrintAttrRows(outf,form,ldap_session,entry,nomatching_attrs,all_attrs,'Attributes not matching schema')

	outf.write('</TABLE>')

      w2lcore.PrintFooter(outf)

  elif command=='vcard':

    from web2ldapcnf import vcard_property
    from charset import utf2iso

    outf.write('Content-type: text/x-vcard; charset=iso-8559-1\n\nbegin:vcard\n')
    vcard_property_keys = vcard_property.keys()
    for vcard_attr in vcard_property_keys:
      attr_list = []
      for attr in vcard_property[vcard_attr][1]:
	if (attr in read_attrs) and (entry.has_key(attr)):
	  if vcard_property[vcard_attr][0]=='cis':
  	    attr_list.append(utf2iso(entry[attr][0]))
	  elif vcard_property[vcard_attr][0]=='mls':
  	    attr_list.append(string.join(string.split(utf2iso(entry[attr][0]),'$'),';'))
      if attr_list:
        outf.write('%s:%s\n' % (vcard_attr,string.join(attr_list,';')))

    outf.write('end:vcard\n')

