########################################################################
# web2ldap
# (c) by Michael Stroeder, michael.stroeder@inka.de
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import string, \
       msbase, cgiforms, charset, ldap, ldapbase, w2lcore, web2ldapcnf

from web2ldapcnf import max_searchparams, script_method
from msbase import intersection#,listremove,union

##############################################################################
# Search for entries and output results as table, pretty-printable output
# or LDIF formatted
##############################################################################

def w2l_Search(outf,command,form):

  # Felder definieren
  form.add(cgiforms.formInputClass('search_filterstr','Search filter string',120,'.*'))
  form.add(cgiforms.formSelectClass('search_mode','Search Mode',[r'(&%s)',r'(|%s)']))
  form.add(cgiforms.formSelectClass('search_scope','Scope',ldapbase.ldap_searchscope_str))
  for i in range(0,max_searchparams):
    form.add(cgiforms.formSelectClass('search_attr','Search attribute',w2lcore.search_attrkeys))
    form.add(cgiforms.formSelectClass('search_option','Search option',w2lcore.search_option.keys()))
    form.add(cgiforms.formInputClass('search_string','Search string',60,'.*'))

  # Felder einlesen und ueberpruefen
  ldap_session = w2lcore.DoParams(command,form)
  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn = ldap_session

  if ldap_url:

    ldap_host,ldap_dn,search_attr,search_scope_str,search_filterstr = parse_ldap_url(ldap_url)

  else:

    # Suchparameter sind alle getrennt angegeben
    if 'ldap_dn' in form.inputkeys:
      ldap_dn = form.field['ldap_dn'][0].content
    else:
      ldap_dn = ''

    # Suchparameter sind alle getrennt angegeben
    if 'search_scope' in form.inputkeys:
      search_scope_str = form.field['search_scope'][0].content
    else:
      search_scope_str = 'sub'

    # LDAP-Filter zusammenbauen
    if 'search_filterstr' in form.inputkeys:
      search_filterstr = form.field['search_filterstr'][0].content
    else:
      if 'search_mode' in form.inputkeys:
	search_mode = form.field['search_mode'][0].content
      else:
	search_mode = '(&%s)'
      search_filter = []
      if ('search_option' in form.inputkeys) and \
         ('search_attr' in form.inputkeys) and \
	 ('search_string' in form.inputkeys):
	for i in range(len(form.field['search_option'])):
	  if form.field['search_string'][i].content:
	    search_filter.append(
	      form.field['search_option'][i].content % (
    		  string.lower(web2ldapcnf.search_attr[form.field['search_attr'][i].content]), \
		  form.field['search_string'][i].content
	      )
	    )
      else:
        raise w2lcore.ErrorExitClass(form,ldap_session,'Invalid or empty search form data.')
      # Teilsuchen werden alle und-verknuepft
      search_filterstr = search_mode % string.join(search_filter,'')
    search_filterstr = w2lcore.input2utf(form.accept_charset,search_filterstr)

  if not ldap_dn:
    ldap_dn = charset.iso2utf(ldap_defaults.get('basedn',''))
    ldap_session = ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_url,ldap_dn

  search_scope = ldapbase.ldap_searchscope.get(search_scope_str,ldap.SCOPE_SUBTREE)

  if command=='print':

    print_template = ldap_defaults.get('print_template','')
    if not print_template:
      raise w2lcore.ErrorExitClass(form,ldap_session,'Print template empty.')
    print_cols = ldap_defaults.get('print_cols','')
    read_attrs = msbase.GrabKeys(print_template).keys
    read_attrs.append('objectclass')

  elif command=='search':

    read_attrs = []

    search_tdtemplate = ldap_defaults.get('search_tdtemplate',{})
    if search_tdtemplate:
      read_attrs = []
      for k in search_tdtemplate.keys():
        for i in msbase.GrabKeys(search_tdtemplate[k]).keys:
	  if not i in read_attrs:
	    read_attrs.append(i)

    search_tablistattrs = ldap_defaults.get('search_tablistattrs',[])
    if search_tablistattrs:
      read_attrs.extend(search_tablistattrs)

    read_attrs.append('objectclass')

  elif command in ['ldif','dsml']:
    # read all attributes
    read_attrs = []

  # Suche ausfhren
  try:
    ldap_msgid = ldap_obj.search(ldap_dn,search_scope,search_filterstr,read_attrs,0)
  except ldap.NO_SUCH_OBJECT:
    result_dnlist = []
  except ldap.FILTER_ERROR:
    raise w2lcore.ErrorExitClass(form,ldap_session,'Bad search filter &quot;%s&quot;.' % w2lcore.utf2display(form.accept_charset,search_filterstr))
  except ldap.SIZELIMIT_EXCEEDED:
    raise w2lcore.ErrorExitClass(form,ldap_session,'Sizelimit exceeded. Please refine search.')
  except:
    exc_obj,exc_value,exc_traceback = sys.exc_info()
    raise w2lcore.ErrorExitClass(form,ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))

  #ldapurl    = scheme "://" [hostport] ["/"
  #                 [dn ["?" [attributes] ["?" [scope]
  #                 ["?" [filter] ["?" extensions]]]]]]
  # Build LDAP URL of search query according to RFC2255
  search_ldap_url = 'ldap://%s/%s?%s?%s?%s' % (
    ldap_host,
    ldap_dn,
    string.join(read_attrs,','),
    search_scope_str,
    search_filterstr,
  )

  if command=='search':

    search_maxhits = ldap_defaults.get('search_maxhits',200)
    result_dnlist = []

    result_type,result_data = ldap_obj.result(ldap_msgid,0)
    if not result_type in ['RES_SEARCH_ENTRY','RES_SEARCH_RESULT']:
      ldap_obj.abandon(ldap_msgid)
      raise w2lcore.ErrorExitClass(form,ldap_session,'Wrong result type: "%s"' % (result_type))

    while result_data and \
          (search_maxhits==0 or (len(result_dnlist)<search_maxhits)):
      result_dnlist.extend(result_data)
      result_type,result_data = ldap_obj.result(ldap_msgid,0)

    if result_data:
      # Only partial results retrieved, undisplayed data avaiable
      ldap_obj.abandon(ldap_msgid)

    result_dnlist.sort()

    # Output search results as table with buttons
    w2lcore.PrintHeader(outf,'Search Results',form.accept_charset)
    w2lcore.MainMenuTable(outf,form,ldap_session)

    if len(result_dnlist):

      w2lcore.CommandTable(outf,[
	w2lcore.CommandButton(form,'print','Printable',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),
	              extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="%s"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="%s">' % (search_scope_str,w2lcore.utf2display(form.accept_charset,search_filterstr)),target='_print'),
	w2lcore.CommandButton(form,'ldif','LDIF',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),
	              extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="%s"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="%s">' % (search_scope_str,w2lcore.utf2display(form.accept_charset,search_filterstr)),target='_altformat'),
	w2lcore.CommandButton(form,'dsml','DSML',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),
	              extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="%s"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="%s">' % (search_scope_str,w2lcore.utf2display(form.accept_charset,search_filterstr)),target='_altformat'),
	w2lcore.CommandButton(form,'filterform','Refine Filter',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,ldap_dn,extrastr='<INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="%s"><INPUT TYPE=HIDDEN NAME="search_scope" VALUE="%s">' % (w2lcore.utf2display(form.accept_charset,search_filterstr),search_scope_str))
      ],div_id='ContextMenuDiv',table_id='ContextMenuTable')

      if result_data:
	outf.write(
          '<P>Only the first %d results of %s search with filter &quot;%s&quot; are displayed. Refine your search.</P>' % (
	    len(result_dnlist),
	    search_scope_str,
	    w2lcore.utf2display(form.accept_charset,search_filterstr)
	  )
	)
      else:
	outf.write(
          '<P>%s search with filter &quot;%s&quot; found %d entries.</P>' % (
	    search_scope_str,
	    w2lcore.utf2display(form.accept_charset,search_filterstr),
	    len(result_dnlist)
	  )
	)

      search_tablistattrs_len = len(search_tablistattrs)

      outf.write('<TABLE ID=SearchResultTable WIDTH="100%%" CELLPADDING=0 CELLSPACING=0 BORDER RULES=rows>\n')

      ldap_binddn_html4 = w2lcore.utf2display(form.accept_charset,ldap_binddn)
      ldap_bindpasswd_html4 = w2lcore.utf2display(form.accept_charset,ldap_bindpasswd)

      for entry in result_dnlist:

	dn = entry[0]
	dn_html4 = w2lcore.utf2display(form.accept_charset,dn)
        objectclasses = entry[1].get('objectclass',[])
	tdtemplate_oc = intersection(objectclasses,search_tdtemplate.keys())

        if tdtemplate_oc:

          # Print entry with the help of pre-defined templates
	  tableentry = msbase.DefaultDict('')
	  attr_list=entry[1].keys()
	  for attr in attr_list:
	    tableentry[attr] = []
	    for value in entry[1][attr]:
  	      tableentry[attr].append(w2lcore.DataStr(form,ldap_session,value))
	    tableentry[attr] = string.join(tableentry[attr],', ')
          tdlist = []
	  for oc in tdtemplate_oc:
	    tdlist.append(search_tdtemplate[oc] % tableentry)
	  tableentrystr='<TD ID=SearchResultItem COLSPAN=%d>%s</TD>\n' % (search_tablistattrs_len,string.join(tdlist,'<BR>\n'))

	elif search_tablistattrs and entry[1].has_key(search_tablistattrs[0]):
          # Print values of attributes listed in search_tablistattrs
	  tableentry=[]
	  attr_list=entry[1].keys()
	  for attr in search_tablistattrs:
            datalist = entry[1].get(attr,[])
            if datalist:
              itemlist = []
	      for item in datalist:
	        itemlist.append(w2lcore.DataStr(form,ldap_session,item))
              tableentry.append(string.join(itemlist,'<BR>'))
	    else:
	      tableentry.append('&nbsp;')
          tableentrystr='<TD ID=SearchResultItem>%s</TD>' % string.join(tableentry,'</TD><TD ID=SearchResultItem>')

	else:
          # Print DN
	  tableentrystr='<TD ID=SearchResultItem COLSPAN=%d>%s</TD>' % (search_tablistattrs_len,w2lcore.DataStr(form,ldap_session,dn))

	outf.write('<TR><TD WIDTH="5%%">%s</TD><TD WIDTH="5%%">%s</TD>%s</TR>' % (
          w2lcore.CommandButton(
	    form,
	    'search',
	    'Go down',
	    ldap_binddn_html4,
	    ldap_bindpasswd_html4,
	    ldap_host,
	    dn_html4,
	    extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="one"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="(objectclass=*)">'
          ),
          w2lcore.CommandButton(
	    form,
	    'read',
	    'Read',
	    ldap_binddn_html4,
	    ldap_bindpasswd_html4,
	    ldap_host,
	    dn_html4
	  ),
	  tableentrystr
	)
      )
      outf.write('</TABLE>')

    else:

      w2lcore.CommandTable(outf,[
	w2lcore.CommandButton(
	  form,
	  'filterform',
	  'Refine Filter',
	  w2lcore.utf2display(form.accept_charset,ldap_binddn),
	  w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
	  ldap_host,
	  w2lcore.utf2display(form.accept_charset,ldap_dn),
	  extrastr='<INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="%s"><INPUT TYPE=HIDDEN NAME="search_scope" VALUE="%s">' % (w2lcore.utf2display(form.accept_charset,search_filterstr),search_scope_str)
        )
      ],div_id='ContextMenuDiv',table_id='ContextMenuTable')
      outf.write('<DIV ID=MessageDiv><P>No Entries found with %s search with filter &quot;%s&quot;.</P></DIV>' % (search_scope_str,w2lcore.utf2display(form.accept_charset,search_filterstr)))

    w2lcore.PrintFooter(outf)

  elif command=='print':

    result_type,result_dnlist = ldap_obj.result(ldap_msgid,1)
    if result_type != 'RES_SEARCH_RESULT':
      raise w2lcore.ErrorExitClass(form,ldap_session,'Wrong result type: "%s"' % (result_type))

    result_dnlist.sort()

    if not result_dnlist:
      raise w2lcore.ErrorExitClass(form,ldap_session,'No search results.')

    # Output search results as pretty-printable table without buttons
    w2lcore.PrintHeader(outf,'Search Results',form.accept_charset)

    table=[]
    for entry in result_dnlist:
      tableentry = msbase.DefaultDict('')
      attr_list=entry[1].keys()
      for attr in attr_list:
        # Output HTML-4 character entities because the output
	# might be saved to a HTML file without
	# Content-type/charset header.
	tableentry[attr] = string.join(w2lcore.utf2display('html4',entry[1][attr]),', ')
      table.append(print_template % tableentry)

    outf.write("""<TABLE ID=PrintTable WIDTH="100%%" CELLPADDING="3%%" CELLSPACING=0 BORDER=1 RULES=rows>
    <COLGROUP SPAN=%d>
    """ % (print_cols))
    tdwidth=100/print_cols
    for i in range(print_cols):
      outf.write('<COL WIDTH="%d%%">\n' % (tdwidth))
    outf.write('</COLGROUP>\n')
    for i in range(len(table)):
      if i%print_cols==0:
        outf.write('<TR>')
      outf.write('<TD WIDTH="%d%%" ALIGN=LEFT VALIGN=top>%s</TD>' % (tdwidth,table[i]))
      if i%print_cols==print_cols-1:
        outf.write('</TR>\n')
    if len(table)%print_cols!=print_cols-1:
      outf.write('</TR>\n')

    outf.write('</TABLE>\n')

    w2lcore.PrintFooter(outf)

  elif command=='ldif':
    import ldif

    result_type,result_dnlist = ldap_obj.result(ldap_msgid,0)
    if result_type !='RES_SEARCH_ENTRY':
      ldap_obj.abandon(ldap_msgid)
      raise w2lcore.ErrorExitClass(form,ldap_session,'Wrong result type: "%s"' % (result_type))

    # Output search results as LDIF
    outf.write('Content-type: text/plain; charset=US-ASCII\n\n')

    while result_dnlist:
      for entry in result_dnlist:
	outf.write('%s\n' % ldif.CreateLDIF(entry[0],entry[1],w2lcore.ldap_binaryattrkeys))
      result_type,result_dnlist = ldap_obj.result(ldap_msgid,0)

  elif command=='dsml':

    import dsml

    result_type,result_dnlist = ldap_obj.result(ldap_msgid,0)
    if result_type !='RES_SEARCH_ENTRY':
      ldap_obj.abandon(ldap_msgid)
      raise w2lcore.ErrorExitClass(form,ldap_session,'Wrong result type: "%s"' % (result_type))

    # Output search results as DSML in a rather primitive way.
    # (level 1 producer)
    outf.write('Content-type: text/xml; charset=utf-8\n\n')
    outf.write("""<dsml:dsml xmlns:dsml="http://www.dsml.org/DSML">
  <!-- directory entries found by %s -->
  <dsml:directory-entries>
""" % (search_ldap_url))

    while result_dnlist:
      for entry in result_dnlist:
	dsml.CreateDSML(outf,entry[0],entry[1],w2lcore.ldap_binaryattrkeys)
      result_type,result_dnlist = ldap_obj.result(ldap_msgid,0)

    outf.write("""  </dsml:directory-entries>
</dsml:dsml>
""")

