########################################################################
# web2ldap
# (c) by Michael Stroeder, michael.stroeder@inka.de
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import sys,os,string,web2ldapcnf
# Modules shipped with web2ldap
import msbase, w2lcore, w2lhandler

######################################################################
# The script was invoked directly => start persistent HTTP handler
######################################################################

import SocketServer,SimpleHTTPServer,BaseHTTPServer,urllib
from socket import gethostname,gethostbyaddr,gethostbyname

class ThreadingHTTPServer(
  SocketServer.ThreadingMixIn,
  BaseHTTPServer.HTTPServer
):
  pass

# Sub-class for serving HTTP requests
class MyHTTPHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):

  def finish(self):
    SimpleHTTPServer.SimpleHTTPRequestHandler.finish(self)
    self.connection.close()

  def is_web2ldap(self):
    return self.path[:9]=='/web2ldap'

  def do_POST(self):
    if self.is_web2ldap():
      self.run_web2ldap()
    else:
      self.send_error(501, "Can only POST to web2ldap.")

  def send_head(self):
    if self.is_web2ldap():
      return self.run_web2ldap()
    else:
      return SimpleHTTPServer.SimpleHTTPRequestHandler.send_head(self)

  def run_web2ldap(self):
      dir, rest = '', self.path[1:]
      i = string.rfind(rest, '?')
      if i >= 0:
          rest, query = rest[:i], rest[i+1:]
      else:
          query = ''
      i = string.find(rest, '/')
      if i >= 0:
          script, rest = rest[:i], rest[i:]
      else:
          script, rest = rest, ''

      env = {}
      if self.headers.typeheader is None:
        env['CONTENT_TYPE'] = self.headers.type
      else:
        env['CONTENT_TYPE'] = self.headers.typeheader
      env['REQUEST_METHOD'] = self.command
      env['SCRIPT_NAME'] = '/web2ldap'
      env['PATH_INFO'] = urllib.unquote(rest)
      env['QUERY_STRING'] = query
      env['SERVER_NAME'] = self.server.server_name
      env['SERVER_PORT'] = str(self.server.server_port)
      env['REMOTE_ADDR'] = self.client_address[0]
      env['REMOTE_PORT'] = self.client_address[1]
      env['DOCUMENT_ROOT'] = '/'
      env['SERVER_SOFTWARE'] = 'webldap %s' % w2lcore.Version
      env['SCRIPT_FILENAME'] = sys.argv[0]
      env['SERVER_ADMIN'] = web2ldapcnf.server_admin
      env['REQUEST_URI'] = '/web2ldap.py'

      for envitem in [
	('Content-length','CONTENT_LENGTH'),
  	('User-Agent','HTTP_USER_AGENT'),
	('Accept','HTTP_ACCEPT'),
	('Accept-Charset','HTTP_ACCEPT_CHARSET'),
	('Accept-Encoding','HTTP_ACCEPT_ENCODING'),
	('Accept-Language','HTTP_ACCEPT_LANGUAGE'),
	('Referer','HTTP_REFERER'),
	('Connection','HTTP_CONNECTION'),
      ]:
        http_header_value = self.headers.getheader(envitem[0])
	if http_header_value:
          env[envitem[1]] = http_header_value

      dispenv = msbase.DefaultDict('')
      dispenv.update(env)

      env['SERVER_SIGNATURE'] = web2ldapcnf.server_signature % dispenv

      self.send_response(200, "web2ldap output follows")

      w2lhandler.HandleHTTPRequest(
	self.rfile,
	self.wfile,
	env
      )
      self.wfile.flush()
      return

def PrintUsage(ErrorMsg=''):
  print """############################################
# web2ldap - a web gateway to LDAP servers #
# (C) 1999,2000 by Michael Stroeder        #
############################################

usage: web2ldap [options]

Options:

-h or -?
    Print out this message

-t on/off
    Run multi-threaded HTTP server.
    If starting multi-threaded fails the script falls
    backs to running a single-threaded HTTP server.
    Default: on

-l [hostname:]port
    Listen to hostname:port. Either hostname:port or
    port is allowed.
    Default: your hostname:4444

-u numeric uid or username
    Switch to given UID or username after binding
    to socket.
    Default: current UID or nobody if started as root.

"""
  if ErrorMsg:
    print '*** Error: %s' % (ErrorMsg)
  sys.exit(1)


def ServerStartup():

  # Set defaults for startup arguments
  web2ldap_server_address = (gethostbyname(gethostname()),4444)
  run_threaded=1
  try:
    current_uid = os.getuid()
    nochuid=0
    try:
      import pwd
      if os.getuid()==0 or os.geteuid()==0:
	uid = pwd.getpwnam('nobody')[2]
      else:
	uid = current_uid
      nopwd = 0
    except ImportError:
      nopwd = 1
  except AttributeError:
    nochuid=1

  # Get startup arguments from command-line options
  import getopt

  try:
    optlist, args=getopt.getopt(sys.argv[1:],"?ht:u:l:")
  except getopt.error,e:
    PrintUsage(str(e))
    sys.exit(1)

  for k,v in optlist:

    if k=="-u":
      try:
	uid = int(v)
      except ValueError:
	if nochuid:
          print 'Changing UID is not available on this system. Aborting.'
	  sys.exit(1)
	if nopwd:
          print 'Module pwd is not available on this system. Aborting.'
	  sys.exit(1)
	try:
	  uid = pwd.getpwnam(v)[2]
	except KeyError:
          PrintUsage('Not a valid user name (option -o).')

    if k=="-l":
      try:
	adr = string.split(v,':')
	if len(adr)==2:
	  web2ldap_server_address = (gethostbyname(adr[0]),string.atoi(adr[1]))
	if len(adr)==1:
	  web2ldap_server_address = (gethostbyname(gethostname()),string.atoi(v))
	else:
	  raise ValueError
      except ValueError:
	PrintUsage('Bind address (option -l) has invalid format.')

    if k=="-t":
      flag = string.lower(v)
      if flag in ['on','off']:
	run_threaded = flag=='on'
      else:
	PrintUsage('Threading option (option -t) must be on or off.')

    if (k=="-h") or (k=="-?"):
      PrintUsage()

  # Change current directory
  os.chdir(os.path.join(os.getcwd(),os.path.dirname(sys.argv[0])))
  os.chdir(os.path.abspath(web2ldapcnf.document_root))

  # Intialize handler class
  HandlerClass = MyHTTPHandler

  # MIME-mapping
  if web2ldapcnf.mime_types:
    # Read mapping from file
    print 'Read MIME-type mapping from file %s.' % (web2ldapcnf.mime_types)
    import mimetypes
    HandlerClass.extensions_map = mimetypes.read_mime_types(web2ldapcnf.mime_types)
    HandlerClass.extensions_map[''] = 'text/plain' # Default, *must* be present
  else:
    # Define very simple default mapping suitable for our needs
    HandlerClass.extensions_map = {
      '': 'text/plain',   # Default, *must* be present
      '.html': 'text/html',
      '.htm': 'text/html',
      '.gif': 'image/gif',
      '.jpg': 'image/jpeg',
      '.jpeg': 'image/jpeg',
      '.css': 'text/css',
    }

  if run_threaded:
    try:
      # Try running multi-threaded
      ServerClass = ThreadingHTTPServer(web2ldap_server_address,HandlerClass)
      print 'Running mode: multi-threaded'
    except:
      # Fall-back running single-threaded
      print 'Running multi-threaded failed!'
      ServerClass = BaseHTTPServer.HTTPServer(web2ldap_server_address,HandlerClass)
      print 'Running mode: fallback to single-threaded'
  else:
    ServerClass = BaseHTTPServer.HTTPServer(web2ldap_server_address,HandlerClass)
    print 'Running mode: single-threaded'

  if (not nochuid) and (current_uid!=uid):
    try:
      print 'Changed to UID %d.' % (uid)
      os.setuid(uid)
    except os.error:
      print 'Error changing to UID %d! Aborting.' % (uid)
      sys.exit(1)

  print """
  Important security note: You started a HTTP server on your
  machine which does not have any security mechanisms for
  protection against unauthorized access!

  Point your favourite browser to

  http://%s:%s/web2ldap

  to access the web2ldap gateway.""" % (
  gethostbyaddr(ServerClass.server_address[0])[0],
  ServerClass.server_address[1]
  )

  ServerClass.serve_forever()
  sys.exit(0)

